<?php

namespace WeiBen\Cases\Providers;

use Illuminate\Support\ServiceProvider;

class CasesServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'cases');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'cases');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        // Bind facade
        $this->app->bind('cases', function ($app) {
            return $this->app->make('WeiBen\Cases\Cases');
        });

                // Bind Cases to repository
        $this->app->bind(
            'WeiBen\Cases\Interfaces\CasesRepositoryInterface',
            \WeiBen\Cases\Repositories\Eloquent\CasesRepository::class
        );

        $this->app->register(\WeiBen\Cases\Providers\AuthServiceProvider::class);
        
        $this->app->register(\WeiBen\Cases\Providers\RouteServiceProvider::class);
                
    }

    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['cases'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('wei-ben/cases.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/cases')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/cases')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
